@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A filter that passes through (unchanged) chunks that contain an integral number
 // of MPEG-2 Transport Stream packets, but returning (in "fDurationInMicroseconds")
 // an updated estimate of the time gap between chunks.
@@ -63,11 +63,15 @@ MPEG2TransportStreamFramer
 }
 
 MPEG2TransportStreamFramer::~MPEG2TransportStreamFramer() {
+  clearPIDStatusTable();
+  delete fPIDStatusTable;
+}
+
+void MPEG2TransportStreamFramer::clearPIDStatusTable() {
   PIDStatus* pidStatus;
   while ((pidStatus = (PIDStatus*)fPIDStatusTable->RemoveNext()) != NULL) {
     delete pidStatus;
   }
-  delete fPIDStatusTable;
 }
 
 void MPEG2TransportStreamFramer::doGetNextFrame() {
@@ -82,11 +86,7 @@ void MPEG2TransportStreamFramer::doStopGettingFrames() {
   FramedFilter::doStopGettingFrames();
   fTSPacketCount = 0;
 
-  // Clear out the existing PID status table:
-  PIDStatus* pidStatus;
-  while ((pidStatus = (PIDStatus*)fPIDStatusTable->RemoveNext()) != NULL) {
-    delete pidStatus;
-  }
+  clearPIDStatusTable();
 }
 
 void MPEG2TransportStreamFramer
