@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that creates new, unicast, "RTPSink"s
 // on demand.
 // Implementation
@@ -90,7 +90,7 @@ OnDemandServerMediaSubsession::sdpLines() {
 
     setSDPLinesFromRTPSink(dummyRTPSink, inputSource);
     Medium::close(dummyRTPSink);
-    Medium::close(inputSource);
+    closeStreamSource(inputSource);
   }
 
   return fSDPLines;
@@ -99,10 +99,10 @@ OnDemandServerMediaSubsession::sdpLines() {
 // A class that represents the state of an ongoing stream
 class StreamState {
 public:
-  StreamState(Port const& serverRTPPort, Port const& serverRTCPPort,
+  StreamState(OnDemandServerMediaSubsession& master,
+              Port const& serverRTPPort, Port const& serverRTCPPort,
 	      RTPSink* rtpSink, BasicUDPSink* udpSink,
-	      float streamDuration, unsigned totalBW, char* CNAME,
-	      FramedSource* mediaSource,
+	      unsigned totalBW, FramedSource* mediaSource,
 	      Groupsock* rtpGS, Groupsock* rtcpGS);
   virtual ~StreamState();
 
@@ -117,13 +117,14 @@ public:
   Port const& serverRTPPort() const { return fServerRTPPort; }
   Port const& serverRTCPPort() const { return fServerRTCPPort; }
 
-  RTPSink const* rtpSink() const { return fRTPSink; }
+  RTPSink* rtpSink() const { return fRTPSink; }
 
   float streamDuration() const { return fStreamDuration; }
 
   FramedSource* mediaSource() const { return fMediaSource; }
 
 private:
+  OnDemandServerMediaSubsession& fMaster;
   Boolean fAreCurrentlyPlaying;
   unsigned fReferenceCount;
 
@@ -133,7 +134,7 @@ private:
   BasicUDPSink* fUDPSink;
 
   float fStreamDuration;
-  unsigned fTotalBW; char* fCNAME; RTCPInstance* fRTCPInstance;
+  unsigned fTotalBW; RTCPInstance* fRTCPInstance;
 
   FramedSource* fMediaSource;
 
@@ -228,8 +229,8 @@ void OnDemandServerMediaSubsession
 
     // Set up the state of the stream.  The stream will get started later:
     streamToken = fLastStreamToken
-      = new StreamState(serverRTPPort, serverRTCPPort, rtpSink, udpSink,
-			duration(), streamBitrate, fCNAME, mediaSource,
+      = new StreamState(*this, serverRTPPort, serverRTCPPort, rtpSink, udpSink,
+			streamBitrate, mediaSource,
 			rtpGroupsock, rtcpGroupsock);
   }
   
@@ -257,7 +258,7 @@ void OnDemandServerMediaSubsession::startStream(unsigned clientSessionId,
 			      rtcpRRHandler, rtcpRRHandlerClientData);
     if (streamState->rtpSink() != NULL) {
       rtpSeqNum = streamState->rtpSink()->currentSeqNo();
-      rtpTimestamp = streamState->rtpSink()->currentTimestamp();
+      rtpTimestamp = streamState->rtpSink()->presetNextTimestamp();
     }
   }
 }
@@ -298,16 +299,17 @@ void OnDemandServerMediaSubsession::setStreamScale(unsigned /*clientSessionId*/,
 
 void OnDemandServerMediaSubsession::deleteStream(unsigned clientSessionId,
 						 void*& streamToken) {
+  StreamState* streamState = (StreamState*)streamToken; 
+
   // Look up (and remove) the destinations for this client session:
   Destinations* destinations
     = (Destinations*)(fDestinationsHashTable->Lookup((char const*)clientSessionId));
   if (destinations != NULL) {
     fDestinationsHashTable->Remove((char const*)clientSessionId);
-  }
 
-  // Stop streaming to these destinations:
-  StreamState* streamState = (StreamState*)streamToken; 
-  if (streamState != NULL) streamState->endPlaying(destinations);
+    // Stop streaming to these destinations:
+    if (streamState != NULL) streamState->endPlaying(destinations);
+  }
 
   // Delete the "StreamState" structure if it's no longer being used:
   if (streamState != NULL && streamState->referenceCount() >= 0) {
@@ -339,6 +341,10 @@ void OnDemandServerMediaSubsession
   // Default implementation: Do nothing
 }
 
+void OnDemandServerMediaSubsession::closeStreamSource(FramedSource *inputSource) {
+  Medium::close(inputSource);
+}
+
 void OnDemandServerMediaSubsession
 ::setSDPLinesFromRTPSink(RTPSink* rtpSink, FramedSource* inputSource) {
   if (rtpSink == NULL) return;
@@ -399,15 +405,15 @@ static void afterPlayingStreamState(void* clientData) {
   // (This can be done only on streams that have a known duration.)
 }
 
-StreamState::StreamState(Port const& serverRTPPort, Port const& serverRTCPPort,
+StreamState::StreamState(OnDemandServerMediaSubsession& master,
+                         Port const& serverRTPPort, Port const& serverRTCPPort,
 			 RTPSink* rtpSink, BasicUDPSink* udpSink,
-			 float streamDuration, unsigned totalBW, char* CNAME,
-			 FramedSource* mediaSource,
+			 unsigned totalBW, FramedSource* mediaSource,
 			 Groupsock* rtpGS, Groupsock* rtcpGS)
-  : fAreCurrentlyPlaying(False), fReferenceCount(1),
+  : fMaster(master), fAreCurrentlyPlaying(False), fReferenceCount(1),
     fServerRTPPort(serverRTPPort), fServerRTCPPort(serverRTCPPort),
-    fRTPSink(rtpSink), fUDPSink(udpSink), fStreamDuration(streamDuration),
-    fTotalBW(totalBW), fCNAME(CNAME), fRTCPInstance(NULL) /* created later */,
+    fRTPSink(rtpSink), fUDPSink(udpSink), fStreamDuration(master.duration()),
+    fTotalBW(totalBW), fRTCPInstance(NULL) /* created later */,
     fMediaSource(mediaSource), fRTPgs(rtpGS), fRTCPgs(rtcpGS) {
 }  
 
@@ -433,7 +439,7 @@ void StreamState
     // Create (and start) a 'RTCP instance' for this RTP sink:
     fRTCPInstance
       = RTCPInstance::createNew(fRTPSink->envir(), fRTCPgs,
-				fTotalBW, (unsigned char*)fCNAME,
+				fTotalBW, (unsigned char*)fMaster.fCNAME,
 				fRTPSink, NULL /* we're a server */);
         // Note: This starts RTCP running automatically
   }
@@ -493,10 +499,8 @@ void StreamState::reclaim() {
   Medium::close(fRTPSink); fRTPSink = NULL;
   Medium::close(fUDPSink); fUDPSink = NULL;
 
-  Medium::close(fMediaSource); fMediaSource = NULL;
+  fMaster.closeStreamSource(fMediaSource); fMediaSource = NULL;
 
   delete fRTPgs; fRTPgs = NULL;
   delete fRTCPgs; fRTCPgs = NULL;
-
-  fReferenceCount = 0;
 }
