@@ -14,55 +14,20 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A generic RTSP client
 // Implementation
 
 #include "RTSPClient.hh"
 #include "RTSPCommon.hh"
 #include "Base64.hh"
+#include "Locale.hh"
 #include <GroupsockHelper.hh>
 #include "our_md5.h"
 #ifdef SUPPORT_REAL_RTSP
 #include "../RealRTSP/include/RealRTSP.hh"
 #endif
 
-// Experimental support for temporarily setting the locale (e.g., to POSIX,
-// for parsing or printing floating-point numbers in protocol headers).
-#ifdef USE_LOCALE
-#include <locale.h>
-#else
-#ifndef LC_NUMERIC
-#define LC_NUMERIC 0
-#endif
-#endif
-
-class Locale {
-public:
-  Locale(char const* newLocale, int category = LC_NUMERIC)
-    : fCategory(category) {
-#ifdef USE_LOCALE
-    fPrevLocale = strDup(setlocale(category, NULL));
-    setlocale(category, newLocale);
-#endif
-  }
-
-  virtual ~Locale() {
-#ifdef USE_LOCALE
-    if (fPrevLocale != NULL) {
-      setlocale(fCategory, fPrevLocale);
-      delete[] fPrevLocale;
-    }
-#endif
-  }
-
-private:
-  int fCategory;
-  char* fPrevLocale;
-};
-
-
-
 ////////// RTSPClient //////////
 
 RTSPClient* RTSPClient::createNew(UsageEnvironment& env,
@@ -201,7 +166,7 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
     char* username; char* password;
     if (authenticator == NULL
 	&& parseRTSPURLUsernamePassword(url, username, password)) {
-      char* result = describeWithPassword(url, username, password);
+      char* result = describeWithPassword(url, username, password, allowKasennaProtocol);
       delete[] username; delete[] password; // they were dynamically allocated
       return result;
     }
@@ -269,11 +234,12 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
       break;
     }
 
-    // Skip every subsequent header line, until we see a blank line
+    // Skip over subsequent header lines, until we see a blank line.
     // The remaining data is assumed to be the SDP descriptor that we want.
-    // We should really do some checking on the headers here - e.g., to
-    // check for "Content-type: application/sdp", "Content-base",
-    // "Content-location", "CSeq", etc. #####
+    // While skipping over the header lines, we also check for certain headers
+    // that we recognize.
+    // (We should also check for "Content-type: application/sdp",
+    // "Content-location", "CSeq", etc.) #####
     char* serverType = new char[fResponseBufferSize]; // ensures enough space
     int contentLength = -1;
     char* lineStart;
@@ -291,6 +257,13 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
 			       lineStart, "\"");
 	  break;
 	}
+      } else if (strncmp(lineStart, "Content-Base:", 13) == 0) {
+	int cbIndex = 13;
+
+	while (lineStart[cbIndex] == ' ' || lineStart[cbIndex] == '\t') ++cbIndex; 
+	if (lineStart[cbIndex] != '\0'/*sanity check*/) {
+	  delete[] fBaseURL; fBaseURL = strDup(&lineStart[cbIndex]);
+	}
       } else if (sscanf(lineStart, "Server: %s", serverType) == 1) {
 	if (strncmp(serverType, "Kasenna", 7) == 0) fServerIsKasenna = True;
 	if (strncmp(serverType, "WMServer", 8) == 0) fServerIsMicrosoft = True;
@@ -305,7 +278,7 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
 		    << redirectionURL << "\"\n";
 	  }
 	  reset();
-	  char* result = describeURL(redirectionURL);
+	  char* result = describeURL(redirectionURL, authenticator, allowKasennaProtocol);
 	  delete[] redirectionURL;
 	  delete[] serverType;
 	  return result;
@@ -491,10 +464,11 @@ char* RTSPClient::describeURL(char const* url, Authenticator* authenticator,
 
 char* RTSPClient
 ::describeWithPassword(char const* url,
-		       char const* username, char const* password) {
+		       char const* username, char const* password,
+		       Boolean allowKasennaProtocol) {
   Authenticator authenticator;
   authenticator.setUsernameAndPassword(username, password);
-  char* describeResult = describeURL(url, &authenticator);
+  char* describeResult = describeURL(url, &authenticator, allowKasennaProtocol);
   if (describeResult != NULL) {
     // We are already authorized
     return describeResult;
@@ -507,7 +481,7 @@ char* RTSPClient
   }
 
   // Try again:
-  describeResult = describeURL(url, &authenticator);
+  describeResult = describeURL(url, &authenticator, allowKasennaProtocol);
   if (describeResult != NULL) {
     // The authenticator worked, so use it in future requests:
     fCurrentAuthenticator = authenticator;
@@ -1016,7 +990,7 @@ static char* createScaleString(float scale, float currentScale) {
     // This is the default value; we don't need a "Scale:" header:
     buf[0] = '\0';
   } else {
-    Locale("POSIX");
+    Locale("POSIX", LC_NUMERIC);
     sprintf(buf, "Scale: %f\r\n", scale);
   }
 
@@ -1030,11 +1004,11 @@ static char* createRangeString(float start, float end) {
     buf[0] = '\0';
   } else if (end < 0) {
     // There's no end time:
-    Locale("POSIX");
+    Locale("POSIX", LC_NUMERIC);
     sprintf(buf, "Range: npt=%.3f-\r\n", start);
   } else {
     // There's both a start and an end time; include them both in the "Range:" hdr
-    Locale("POSIX");
+    Locale("POSIX", LC_NUMERIC);
     sprintf(buf, "Range: npt=%.3f-%.3f\r\n", start, end);
   }
 
@@ -2273,7 +2247,7 @@ Boolean RTSPClient::parseScaleHeader(char const* line, float& scale) {
   if (_strncasecmp(line, "Scale: ", 7) != 0) return False;
   line += 7;
 
-  Locale("POSIX");
+  Locale("POSIX", LC_NUMERIC);
   return sscanf(line, "%f", &scale) == 1;
 }
 
@@ -2317,6 +2291,8 @@ Boolean RTSPClient::parseGetParameterHeader(char const* line,
 
 Boolean RTSPClient::setupHTTPTunneling(char const* urlSuffix,
 				       Authenticator* authenticator) {
+  // Set up RTSP-over-HTTP tunneling, as described in
+  //     http://developer.apple.com/documentation/QuickTime/QTSS/Concepts/chapter_2_section_14.html
   if (fVerbosityLevel >= 1) {
     envir() << "Requesting RTSP-over-HTTP tunneling (on port "
 	    << fTunnelOverHTTPPortNum << ")\n\n";
@@ -2440,6 +2416,7 @@ void RTSPClient::incomingRequestHandler1() {
   bytesRead = getResponse1(readBuf, fResponseBufferSize);
   if (bytesRead == 0) {
     envir().setResultErrMsg("Failed to read response: ");
+    envir().taskScheduler().turnOffBackgroundReadHandling(fInputSocketNum); // because the connection died
     return;
   }
   // Parse the request string into command name and 'CSeq',
