@@ -14,7 +14,7 @@ along with this library; if not, write to the Free Software Foundation, Inc.,
 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 **********/
 // "liveMedia"
-// Copyright (c) 1996-2006 Live Networks, Inc.  All rights reserved.
+// Copyright (c) 1996-2007 Live Networks, Inc.  All rights reserved.
 // A 'ServerMediaSubsession' object that creates new, unicast, "RTPSink"s
 // on demand, from an WAV audio file.
 // Implementation
@@ -163,14 +163,16 @@ RTPSink* WAVAudioFileServerMediaSubsession
       } else {
 	payloadFormatCode = rtpPayloadTypeIfDynamic;
       }
-    } else {
+    } else if (fAudioFormat == WA_PCMA) {
       mimeType = "PCMA";
       if (fSamplingFrequency == 8000 && fNumChannels == 1) {
 	payloadFormatCode = 8; // a static RTP payload type
       } else {
 	payloadFormatCode = rtpPayloadTypeIfDynamic;
       }
-    }
+    } else { //unknown format
+		break;
+	}
     
     return SimpleRTPSink::createNew(envir(), rtpGroupsock,
 				    payloadFormatCode, fSamplingFrequency,
